#! /usr/bin/env python3

import sys
import argparse

parser = argparse.ArgumentParser(
    prog = 'summarizeHaplobocks.py',
    description = 'Create a table summarizing the haplotypes generated by HapCut2. Writes to stdout.',
    usage = "summarizeHaploblocks.py input > output",
    exit_on_error = False
    )
parser.add_argument('input', help = "Input HapCut2 .blocks file.")

if len(sys.argv) == 1:
    parser.print_help(sys.stderr)
    sys.exit(1)

args = parser.parse_args()

with open(args.input, "r") as f:
    print("contig\tstart\tend\tn_snps", file = sys.stdout)
    while True:
        line = f.readline()
        if not line:
            break
        line = line.strip().split()

        # identify if the line is a BLOCK line, asterisks, or SNP
        try:
            if line[0] == "BLOCK:":
                header = 0
            elif "*" in line[0]:
                continue
            else:
                header += 1
        except:
            break
        if header == 0:
            n      = line[6]
            span   = line[8]
        elif header == 1:
            chrm   = line[3]
            start  = line[4]
            end    = int(start) + int(span)
            print(f"{chrm}\t{start}\t{end}\t{n}", file = sys.stdout)
        else:
            # skip all the snps after the first since we have all the info we need
            continue
    